%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [C] = Newton(wmri, umri, velocidadeEntrada, NE, NI, NS, NDE, NDI, NELE, NDoF, ASSMtrx, DomNodeID, X, ...
                      Y, ro, mi, lambda)

    itermax = 1e2;              
    E = 1e-4;
     
    [iuldof, ivldof, ipldof] = grauliberdade();

    C = zeros(NDoF,1);
    C0 = zeros(NDoF,1);

    % Global Residue Vector
    [Rv, bv] = formRv(wmri, umri, velocidadeEntrada, NE, NI, NS, NDE, NDI, NELE, NDoF, DomNodeID, ASSMtrx, ...,
                  iuldof, ivldof, ipldof, X, Y, ro, mi, C);
    rvnorm = norm(Rv);
    oldrvnorm=1;
    
    iter = 1;
    
    % Newton's Method
    while  ( (rvnorm > E) && (iter < itermax) )
       disp(sprintf('Iteration = %d', iter));
       disp(sprintf('Norm = %4.4f', rvnorm));
        
       % Global Jacobian Matrix Calculation
       [J, S] = formJ(wmri, umri, NE, NI, NS, NDE, NDI, NELE, NDoF, DomNodeID, ASSMtrx, ...
                  iuldof,ivldof,ipldof, X, Y, ro, mi, C);  
       b = -Rv;
       
       S(S~=0) = 1;     
       
       [LSP, USP] = lu(J'*J+lambda*(S')*S);       
       C = bicgstab(J'*J+lambda*(S')*S, J'*(J*C+b)+lambda*S'*bv, 1e-12, 1e6, LSP, USP);
      
       disp(sprintf('|S*c_i-u_mri| = %4.4f',norm(S*C-bv,2)));
       disp(sprintf('|J*dc_i+R| = %4.4f',norm(J*C-(J*C0+b),2)));
       display(' ');
       
       [Rv, bv] = formRv(wmri, umri, velocidadeEntrada, NE, NI, NS, NDE, NDI, NELE, NDoF, DomNodeID, ASSMtrx, ...,
                  iuldof, ivldof, ipldof, X, Y, ro, mi, C);
       rvnorm = norm(C-C0,2);

       oldrvnorm = rvnorm;
       C0 = C;
       iter = iter + 1;
       
    end

    if (oldrvnorm > E) 
        display('Did not converge');
    end
    
    SNR = 10*log10(norm(bv)/norm(S*C-bv,2));
    disp(sprintf('SER = %4.2f',SNR));
end

   
