%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Phi, GradPhi, Psi] = basifunc (XI, NETA)
    % Function that returns  Phi, Eta, dPhi e dEta on 3 Gauss points.

    % Gauss Points (XI: horizontal direction, NETA: vertical direction) :

    % Vector Phi (nove rows and one column)
    Phi = [((XI)*(XI-1)*(NETA)*(NETA-1))/4;...
    ((XI)*(XI+1)*(NETA)*(NETA -1))/4;...
    ((XI)*(XI+1)*(NETA)*(NETA+1))/4;...
    ((XI)*(XI-1)*(NETA)*(NETA+1))/4;...
    ((XI-1)*(XI+1)*(NETA)*(NETA-1))/(-2);...
    ((XI)*(XI+1)*(NETA-1)*(NETA+1))/(-2);...
    ((XI+1)*(XI-1)*(NETA)*(NETA+1))/(-2);...
    ((XI)*(XI-1)*(NETA+1)*(NETA-1))/(-2);...
    ((XI+1)*(XI-1)*(NETA-1)*(NETA+1))];


    % Matrix GradPhi (9 columns and 2 rows)   
    GradPhi=[((2*XI- 1)*((NETA)^2-NETA))/4,((2*XI + 1)*((NETA)^2 - NETA))/4,...
      ((2*XI + 1)*((NETA)^2 + NETA))/4, ((2*XI - 1)*((NETA)^2 + NETA))/4,... 
      ((2*XI)*((NETA)^2 - NETA))/(-2), ((2*XI + 1)*((NETA)^2 - 1))/(-2),... 
      ((2*XI)*((NETA)^2 + NETA))/(-2),  ((2*XI- 1)*((NETA)^2 - 1))/(-2),... 
      ((2*XI)*((NETA)^2 - 1));... 
      (((XI)^2 - XI)*(2*NETA -1))/4, ((XI)^2 + XI)*(2*NETA -1)/4,... 
      ((XI)^2 + XI)*(2*NETA +1)/4, ((XI)^2 - XI)*(2*NETA +1)/4,... 
      ((XI)^2 - 1)*(2*NETA- 1)/(-2), ((XI)^2 + XI)*(2*NETA)/(-2),... 
      ((XI)^2 - 1)*(2*NETA +1)/(-2), ((XI)^2 - XI)*(2*NETA)/(-2),...
      ((XI)^2 - 1)*(2*NETA)];

    % Vector Psi (3 rows and one column)
    Psi = [ 1; NETA; XI ];

end

