%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [coordGlobal, conectMatrix, malhaNodesFixed, contornoQuad, coordGlobalQuad, conectMatrixQuad, malhaNodesQuad, malhaVoxelsQuad] = geraMalha(malhaVoxels, nVoxelsX, nVoxelsY)
    
    %Dimensoes dos Voxels (mm)
    dx = 1.0;
    dy = 0.5;
    
    %Dimensoes da Malha (mm)
    vX = nVoxelsX;
    vY = nVoxelsY;
         
    countOnes = zeros(vX,1);  
    malhaNodesFixed = zeros(vY,vX);
    malhaVoxelsQuad = zeros(2*vY,2*vX-1);
    malhaNodesQuad = zeros(2*vY,2*vX-1);
    
    %Coloca nos entre os 4 nos
    for j = 1:vX
        for i = 1:vY
            if malhaVoxels(i, j) ~= 0
                malhaVoxelsQuad(2*i-1, 2*j-1) = 1;
            end
        end
    end
    
    for j = 1:2*vX-3
        for i = 1:2*vY
            if malhaVoxelsQuad(i, j) ~= 0 && malhaVoxelsQuad(i, j + 2) ~= 0
                malhaVoxelsQuad(i, j+1) = 1;
            end
        end
    end
    
    for i = 1:2*vY-2
        for j = 1:2*vX-1
            if malhaVoxelsQuad(i, j) ~= 0 && malhaVoxelsQuad(i + 2, j) ~= 0
                malhaVoxelsQuad(i+1, j) = 1;
            end
        end
    end
          
    contornoQuad = zeros(2*vY,2*vX-1);
    %Verifica qual o contorno da malha interna
    %Varreduta Horizontal
    for i = 1:2*vY
        for j = 1:2*vX-2
            if malhaVoxelsQuad(i, j) == 0 && malhaVoxelsQuad(i, j+1) == 1
                contornoQuad(i, j+1) = 1;
            end
            if malhaVoxelsQuad(i, j) == 1 && malhaVoxelsQuad(i, j+1) == 0
                contornoQuad(i, j) = 1;
            end
            if j == 1 && malhaVoxelsQuad(i, j) == 1
                contornoQuad(i, j) = 1;
            end
            if j == 2*vX-2 && malhaVoxelsQuad(i, j+1) == 1
                contornoQuad(i, j+1) = 1;
            end
        end
    end
    
    %Varreduta Vertical
    for j = 1:2*vX-1
        for i = 1:2*vY-1
            if malhaVoxelsQuad(i, j) == 0 && malhaVoxelsQuad(i+1, j) == 1
                contornoQuad(i+1, j) = 1;
            end
            if malhaVoxelsQuad(i, j) == 1 && malhaVoxelsQuad(i+1, j) == 0
                contornoQuad(i, j) = 1;
            end
        end
    end
       
    qtc = 0;
    for j = 1:2*vX-1
        for i = 1:2*vY
            if contornoQuad(i, j) == 1
                qtc = qtc + 1; 
            end
        end
    end
        
    
    %Calcula coordenadas do mapa de magnitude
    nodes = 1;
    for j = 1:vX
        countOnes(j, 1) = 0;
        for i = 1:vY
            if malhaVoxels(i, j) ~= 0
                coordGlobalVoxels(nodes, 1) = (j-1)*dx;
                coordGlobalVoxels(nodes, 2) = (vY-i)*dy;
                coordGlobal(nodes, 1) = dx/2 + (j-1)*dx;
                coordGlobal(nodes, 2) = dy/2 + (vY-i)*dy;
                malhaNodesFixed(i, j) = nodes;
                malhaNodesQuad(2*i-1, 2*j-1) = nodes;
                nodes = nodes + 1;
                countOnes(j, 1) = countOnes(j, 1) + 1;
            end
        end
    end
          
    %Define o primeiro no local da matriz de conectividade para cada elemento
    iEle = 1;
    for j = 1:vX-1
        for i = 1:vY-1
            if malhaVoxels(i, j) ~= 0 && malhaVoxels(i + 1, j) ~= 0
                conectMatrix(iEle, 1) = malhaNodesFixed(i, j);
                iEle = iEle + 1;
            end
        end
    end

    %Define os outros nos locais em funcao do primeiro 
    iEle = 1;
    for j = 1:vX-1
        for i = 1:vY-1
            if malhaVoxels(i, j) ~= 0 && malhaVoxels(i + 1, j) ~= 0
                conectMatrix(iEle, 2) = conectMatrix(iEle, 1) + 1;
                conectMatrix(iEle, 3) = conectMatrix(iEle, 1) + countOnes(j,1) + 1;
                conectMatrix(iEle, 4) = conectMatrix(iEle, 1) + countOnes(j,1);
                iEle = iEle + 1;
            end
        end
    end
    
    %Elimina elementos problematicos
    conectMatrix(iEle-1, :) = [];
    conectMatrix(iEle-2, :) = [];
 
    %Atualiza quantidade de elementos e nos
    ele = size(conectMatrix, 1);
    nodes = size(coordGlobal, 1);
    
    %Funcao fixMalha reorganiza conectMatrix
    fixedMatrix = fixMalha(ele, coordGlobal, conectMatrix);
    conectMatrix = fixedMatrix;
    
    %Atualiza quantidade de elementos
    ele = size(conectMatrix, 1);
    
    %Calcula coordenadas da malha final com elementos quadrangulares
    countNodesQuad = zeros(2*vX,1);  
    
    nodesQuad = 1;
    for j = 1:2*vX-1
        for i = 1:2*vY
            if malhaVoxelsQuad(i, j) ~= 0
                %coordGlobalQuad(nodesQuad, 1) = dx/2 + (j-1)*dx;
                %coordGlobalQuad(nodesQuad, 2) =(2*vY-i)*dy;
                %nodesQuad = nodesQuad + 1;
            end
        end
    end
    
    nodesQuad = nodes + 1;
    for j = 1:2*vX-1
        countNodesQuad(j, 1) = 0;
        for i = 1:2*vY
            if malhaVoxelsQuad(i, j) ~= 0
                %Nos locals 5 e 7
                if malhaNodesQuad(i, j) == 0 && mod(j, 2) == 1 %colunas impares
                    malhaNodesQuad(i, j) = nodesQuad;
                    nodesQuad = nodesQuad + 1;
                end
                countNodesQuad(j, 1) = countNodesQuad(j, 1) + 1;
            end
        end
    end
    
    %Nos locais 8 e 6
    for j = 2:2:2*vX-1
        for i = 1:2:2*vY
            if malhaVoxelsQuad(i, j) ~= 0
                if malhaNodesQuad(i, j) == 0 && mod(j, 2) == 0 %colunas pares
                    malhaNodesQuad(i, j) = nodesQuad;
                    nodesQuad = nodesQuad + 1;
                end
            end
        end
    end
    
    %Nos centrais 9
    for j = 2:2:2*vX-1
        for i = 2:2:2*vY
            if malhaVoxelsQuad(i, j) ~= 0
                if malhaNodesQuad(i, j) == 0 && mod(j, 2) == 0 %colunas pares
                    malhaNodesQuad(i, j) = nodesQuad;
                    nodesQuad = nodesQuad + 1;
                end
            end
        end
    end

    %Nos locais 1, 2, 3 e 4
    conectMatrixQuad(:, 1) = conectMatrix(:, 1);
    conectMatrixQuad(:, 2) = conectMatrix(:, 2);
    conectMatrixQuad(:, 3) = conectMatrix(:, 3);
    conectMatrixQuad(:, 4) = conectMatrix(:, 4);
    
    %No local 5
    iEleQuad = 1;
    for j = 1:2:2*vX-2
        for i = 2:2:2*vY-2
            if malhaNodesQuad(i, j) ~= 0 && malhaNodesQuad(i, j+2) ~= 0
                conectMatrixQuad(iEleQuad, 5) = malhaNodesQuad(i, j);
                iEleQuad = iEleQuad + 1;
            end
        end
    end
    conectMatrixQuad(iEleQuad-1, :) = [];
    
    %No local 7
    iEleQuad = 1;
    for j = 3:2:2*vX-1
        for i = 2:2:2*vY-2
            if malhaNodesQuad(i, j) ~= 0 && malhaNodesQuad(i, j-2) ~= 0
                conectMatrixQuad(iEleQuad, 7) = malhaNodesQuad(i, j);
                iEleQuad = iEleQuad + 1;
            end
        end
    end
    conectMatrixQuad(iEleQuad-1, :) = [];
    
    %Nos locais 6 e 8
    iEleQuad = 1;
    for j = 2:2:2*vX-2
        for i = 1:2:2*vY-2
            if malhaVoxelsQuad(i, j) ~= 0 && malhaVoxelsQuad(i + 2, j) ~= 0
                conectMatrixQuad(iEleQuad, 8) = malhaNodesQuad(i, j);
                conectMatrixQuad(iEleQuad, 6) = malhaNodesQuad(i, j) + 1;
                iEleQuad = iEleQuad + 1;
            end
        end
    end
    conectMatrixQuad(iEleQuad-1, :) = [];
    
    %No local 9
    iEleQuad = 1;
    for j = 2:2:2*vX-2
        for i = 2:2:2*vY-2
            if malhaNodesQuad(i, j) ~= 0
                conectMatrixQuad(iEleQuad, 9) = malhaNodesQuad(i, j);
                iEleQuad = iEleQuad + 1;
            end
        end
    end
    
    %Adiciona ultimo elemento
    conectMatrixQuad(iEleQuad-1, :) = [749 750 772 771 1474 2213 1494 2212 2883];
    
    %Coordenadas dos novos nos
    %Coordenadas dos nos fixos
    nodesQuad = 1;
    for i = 1:size(coordGlobal,1)
        coordGlobalQuad(nodesQuad, 1) = coordGlobal(i, 1);
        coordGlobalQuad(nodesQuad, 2) = coordGlobal(i, 2);
        nodesQuad = nodesQuad + 1;
    end
    nodesQuad = nodesQuad - 1;
    
    %Coordenadas X
    for j = 1:2:2*vX-3
        for i = 1:2:2*vY-1
            if malhaVoxelsQuad(i, j) ~= 0 && malhaVoxelsQuad(i, j+1) ~= 0 && malhaVoxelsQuad(i, j+2) ~= 0
                globalNode1 = malhaNodesQuad(i, j);
                globalNode2 = malhaNodesQuad(i, j+2);
                nodesQuad = malhaNodesQuad(i, j+1);
                X1 = coordGlobal(globalNode1, 1);
                X2 = coordGlobal(globalNode2, 1);
                Y1 = coordGlobal(globalNode1, 2);
                Y2 = coordGlobal(globalNode2, 2);
                coordGlobalQuad(nodesQuad, 1) = (X1 + X2)/2;
                coordGlobalQuad(nodesQuad, 2) = (Y1 + Y2)/2;
            end
        end
    end
    
    %Coordenadas Y
    for i = 1:2:2*vY-2
        for j = 1:2:2*vX-1
            if malhaVoxelsQuad(i, j) ~= 0 && malhaVoxelsQuad(i+1, j) ~= 0 && malhaVoxelsQuad(i+2, j) ~= 0
                globalNode1 = malhaNodesQuad(i, j);
                globalNode2 = malhaNodesQuad(i+2, j);
                nodesQuad = malhaNodesQuad(i+1, j);
                X1 = coordGlobal(globalNode1, 1);
                X2 = coordGlobal(globalNode2, 1);
                Y1 = coordGlobal(globalNode1, 2);
                Y2 = coordGlobal(globalNode2, 2);
                coordGlobalQuad(nodesQuad, 1) = (X1 + X2)/2;
                coordGlobalQuad(nodesQuad, 2) = (Y1 + Y2)/2;
            end
        end
    end
    
    %Coordenadas dos nos centrais
    for j = 2:2:2*vX-1
        for i = 2:2:2*vY-1
            if malhaVoxelsQuad(i, j-1) ~= 0 && malhaVoxelsQuad(i, j) ~= 0 && malhaVoxelsQuad(i, j+1) ~= 0
                globalNode1 = malhaNodesQuad(i, j-1);
                globalNode2 = malhaNodesQuad(i, j+1);
                nodesQuad = malhaNodesQuad(i, j);
                X1 = coordGlobalQuad(globalNode1, 1);
                X2 = coordGlobalQuad(globalNode2, 1);
                Y1 = coordGlobalQuad(globalNode1, 2);
                Y2 = coordGlobalQuad(globalNode2, 2);
                coordGlobalQuad(nodesQuad, 1) = (X1 + X2)/2;
                coordGlobalQuad(nodesQuad, 2) = (Y1 + Y2)/2;
            end
        end
    end
    
end